// Generated by `wit-bindgen` 0.51.0. DO NOT EDIT!
// Options used:
//   * with "wasi:cli/environment@0.3.0-rc-2026-01-06" = "crate::cli::environment"
//   * with "wasi:cli/exit@0.3.0-rc-2026-01-06" = "crate::cli::exit"
//   * with "wasi:cli/stdin@0.3.0-rc-2026-01-06" = "crate::cli::stdin"
//   * with "wasi:cli/stdout@0.3.0-rc-2026-01-06" = "crate::cli::stdout"
//   * with "wasi:cli/stderr@0.3.0-rc-2026-01-06" = "crate::cli::stderr"
//   * with "wasi:cli/types@0.3.0-rc-2026-01-06" = "crate::cli::types"
//   * with "wasi:cli/terminal-input@0.3.0-rc-2026-01-06" = "crate::cli::terminal_input"
//   * with "wasi:cli/terminal-output@0.3.0-rc-2026-01-06" = "crate::cli::terminal_output"
//   * with "wasi:cli/terminal-stdin@0.3.0-rc-2026-01-06" = "crate::cli::terminal_stdin"
//   * with "wasi:cli/terminal-stdout@0.3.0-rc-2026-01-06" = "crate::cli::terminal_stdout"
//   * with "wasi:cli/terminal-stderr@0.3.0-rc-2026-01-06" = "crate::cli::terminal_stderr"
//   * with "wasi:clocks/monotonic-clock@0.3.0-rc-2026-01-06" = "crate::clocks::monotonic_clock"
//   * with "wasi:clocks/system-clock@0.3.0-rc-2026-01-06" = "crate::clocks::system_clock"
//   * with "wasi:clocks/types@0.3.0-rc-2026-01-06" = "crate::clocks::types"
//   * with "wasi:filesystem/types@0.3.0-rc-2026-01-06" = "crate::filesystem::types"
//   * with "wasi:filesystem/preopens@0.3.0-rc-2026-01-06" = "crate::filesystem::preopens"
//   * with "wasi:random/random@0.3.0-rc-2026-01-06" = "crate::random::random"
//   * with "wasi:random/insecure@0.3.0-rc-2026-01-06" = "crate::random::insecure"
//   * with "wasi:random/insecure-seed@0.3.0-rc-2026-01-06" = "crate::random::insecure_seed"
//   * with "wasi:sockets/types@0.3.0-rc-2026-01-06" = "crate::sockets::types"
//   * with "wasi:sockets/ip-name-lookup@0.3.0-rc-2026-01-06" = "crate::sockets::ip_name_lookup"
//   * type_section_suffix: "rust-wasip3-0.4.0+wasi-0.3.0-rc-2026-01-06-from-crates-io-command-world"
//   * default-bindings-module: "$crate"
//   * export-macro-name: _export_command
//   * pub-export-macro
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::cli::environment as __with_name0;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::cli::exit as __with_name1;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::cli::types as __with_name2;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::cli::stdin as __with_name3;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::cli::stdout as __with_name4;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::cli::stderr as __with_name5;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::cli::terminal_input as __with_name6;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::cli::terminal_output as __with_name7;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::cli::terminal_stdin as __with_name8;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::cli::terminal_stdout as __with_name9;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::cli::terminal_stderr as __with_name10;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::clocks::types as __with_name11;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::clocks::monotonic_clock as __with_name12;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::clocks::system_clock as __with_name13;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::filesystem::types as __with_name14;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::filesystem::preopens as __with_name15;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::sockets::types as __with_name16;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::sockets::ip_name_lookup as __with_name17;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::random::random as __with_name18;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::random::insecure as __with_name19;
#[allow(unfulfilled_lint_expectations, unused_imports)]
use crate::random::insecure_seed as __with_name20;
#[rustfmt::skip]
#[allow(dead_code, clippy::all)]
pub mod exports {
    pub mod wasi {
        pub mod cli {
            #[allow(dead_code, async_fn_in_trait, unused_imports, clippy::all)]
            pub mod run {
                #[used]
                #[doc(hidden)]
                static __FORCE_SECTION_REF: fn() = super::super::super::super::__link_custom_section_describing_imports;
                use super::super::super::super::_rt;
                #[doc(hidden)]
                #[allow(non_snake_case, unused_unsafe)]
                pub unsafe fn _export_run_cabi<T: Guest>() -> i32 {
                    unsafe {
                        #[cfg(target_arch = "wasm32")] _rt::run_ctors_once();
                        wit_bindgen::rt::async_support::start_task(async move {
                            let _task_cancel = wit_bindgen::rt::async_support::TaskCancelOnDrop::new();
                            let result0 = &{ T::run().await };
                            let result1 = match result0 {
                                Ok(_) => 0i32,
                                Err(_) => 1i32,
                            };
                            #[cfg(target_arch = "wasm32")]
                            #[link(
                                wasm_import_module = "[export]wasi:cli/run@0.3.0-rc-2026-01-06"
                            )]
                            unsafe extern "C" {
                                #[link_name = "[task-return]run"]
                                fn wit_import2(_: i32);
                            }
                            #[cfg(not(target_arch = "wasm32"))]
                            unsafe extern "C" fn wit_import2(_: i32) {
                                unreachable!()
                            }
                            _task_cancel.forget();
                            wit_import2(result1);
                        })
                    }
                }
                #[doc(hidden)]
                #[allow(non_snake_case)]
                pub unsafe fn __callback_run(
                    event0: u32,
                    event1: u32,
                    event2: u32,
                ) -> u32 {
                    unsafe {
                        wit_bindgen::rt::async_support::callback(event0, event1, event2)
                    }
                }
                pub trait Guest {
                    /// Run the program.
                    #[allow(async_fn_in_trait)]
                    async fn run() -> Result<(), ()>;
                }
                #[doc(hidden)]
                #[macro_export]
                macro_rules! __export_wasi_cli_run_0_3_0_rc_2026_01_06_cabi {
                    ($ty:ident with_types_in $($path_to_types:tt)*) => {
                        const _ : () = { #[unsafe (export_name =
                        "[async-lift]wasi:cli/run@0.3.0-rc-2026-01-06#run")] unsafe
                        extern "C" fn export_run() -> i32 { unsafe { $($path_to_types)*::
                        _export_run_cabi::<$ty > () } } #[unsafe (export_name =
                        "[callback][async-lift]wasi:cli/run@0.3.0-rc-2026-01-06#run")]
                        unsafe extern "C" fn _callback_run(event0 : u32, event1 : u32,
                        event2 : u32) -> u32 { unsafe { $($path_to_types)*::
                        __callback_run(event0, event1, event2) } } };
                    };
                }
                #[doc(hidden)]
                pub use __export_wasi_cli_run_0_3_0_rc_2026_01_06_cabi;
            }
        }
    }
}
#[rustfmt::skip]
mod _rt {
    #![allow(dead_code, unused_imports, clippy::all)]
    #[cfg(target_arch = "wasm32")]
    pub fn run_ctors_once() {
        wit_bindgen::rt::run_ctors_once();
    }
}
/// Generates `#[unsafe(no_mangle)]` functions to export the specified type as
/// the root implementation of all generated traits.
///
/// For more information see the documentation of `wit_bindgen::generate!`.
///
/// ```rust
/// # macro_rules! _export_command{ ($($t:tt)*) => (); }
/// # trait Guest {}
/// struct MyType;
///
/// impl Guest for MyType {
///     // ...
/// }
///
/// _export_command!(MyType);
/// ```
#[allow(unused_macros)]
#[doc(hidden)]
#[macro_export]
macro_rules! __export_command_impl {
    ($ty:ident) => {
        $crate::_export_command!($ty with_types_in $crate);
    };
    ($ty:ident with_types_in $($path_to_types_root:tt)*) => {
        $($path_to_types_root)*::
        exports::wasi::cli::run::__export_wasi_cli_run_0_3_0_rc_2026_01_06_cabi!($ty
        with_types_in $($path_to_types_root)*:: exports::wasi::cli::run); const _ : () =
        { #[rustfmt::skip] #[cfg(target_arch = "wasm32")] #[unsafe (link_section =
        "component-type:wit-bindgen:0.51.0:wasi:cli@0.3.0-rc-2026-01-06:command:imports and exportsrust-wasip3-0.4.0+wasi-0.3.0-rc-2026-01-06-from-crates-io-command-world")]
        #[doc(hidden)] #[allow(clippy::octal_escapes)] pub static
        __WIT_BINDGEN_COMPONENT_TYPE : [u8; 8040] = *
        b"\
\0asm\x0d\0\x01\0\0\x19\x16wit-component-encoding\x04\0\x07\xea=\x01A\x02\x01A3\x01\
B\x0a\x01o\x02ss\x01p\0\x01@\0\0\x01\x04\0\x0fget-environment\x01\x02\x01ps\x01@\
\0\0\x03\x04\0\x0dget-arguments\x01\x04\x01ks\x01@\0\0\x05\x04\0\x0fget-initial-\
cwd\x01\x06\x03\0(wasi:cli/environment@0.3.0-rc-2026-01-06\x05\0\x01B\x03\x01j\0\
\0\x01@\x01\x06status\0\x01\0\x04\0\x04exit\x01\x01\x03\0!wasi:cli/exit@0.3.0-rc\
-2026-01-06\x05\x01\x01B\x02\x01m\x03\x02io\x15illegal-byte-sequence\x04pipe\x04\
\0\x0aerror-code\x03\0\0\x03\0\"wasi:cli/types@0.3.0-rc-2026-01-06\x05\x02\x02\x03\
\0\x02\x0aerror-code\x01B\x08\x02\x03\x02\x01\x03\x04\0\x0aerror-code\x03\0\0\x01\
f\x01}\x01j\0\x01\x01\x01e\x01\x03\x01o\x02\x02\x04\x01@\0\0\x05\x04\0\x0fread-v\
ia-stream\x01\x06\x03\0\"wasi:cli/stdin@0.3.0-rc-2026-01-06\x05\x04\x01B\x06\x02\
\x03\x02\x01\x03\x04\0\x0aerror-code\x03\0\0\x01f\x01}\x01j\0\x01\x01\x01C\x01\x04\
data\x02\0\x03\x04\0\x10write-via-stream\x01\x04\x03\0#wasi:cli/stdout@0.3.0-rc-\
2026-01-06\x05\x05\x01B\x06\x02\x03\x02\x01\x03\x04\0\x0aerror-code\x03\0\0\x01f\
\x01}\x01j\0\x01\x01\x01C\x01\x04data\x02\0\x03\x04\0\x10write-via-stream\x01\x04\
\x03\0#wasi:cli/stderr@0.3.0-rc-2026-01-06\x05\x06\x01B\x01\x04\0\x0eterminal-in\
put\x03\x01\x03\0+wasi:cli/terminal-input@0.3.0-rc-2026-01-06\x05\x07\x01B\x01\x04\
\0\x0fterminal-output\x03\x01\x03\0,wasi:cli/terminal-output@0.3.0-rc-2026-01-06\
\x05\x08\x02\x03\0\x06\x0eterminal-input\x01B\x06\x02\x03\x02\x01\x09\x04\0\x0et\
erminal-input\x03\0\0\x01i\x01\x01k\x02\x01@\0\0\x03\x04\0\x12get-terminal-stdin\
\x01\x04\x03\0+wasi:cli/terminal-stdin@0.3.0-rc-2026-01-06\x05\x0a\x02\x03\0\x07\
\x0fterminal-output\x01B\x06\x02\x03\x02\x01\x0b\x04\0\x0fterminal-output\x03\0\0\
\x01i\x01\x01k\x02\x01@\0\0\x03\x04\0\x13get-terminal-stdout\x01\x04\x03\0,wasi:\
cli/terminal-stdout@0.3.0-rc-2026-01-06\x05\x0c\x01B\x06\x02\x03\x02\x01\x0b\x04\
\0\x0fterminal-output\x03\0\0\x01i\x01\x01k\x02\x01@\0\0\x03\x04\0\x13get-termin\
al-stderr\x01\x04\x03\0,wasi:cli/terminal-stderr@0.3.0-rc-2026-01-06\x05\x0d\x01\
B\x02\x01w\x04\0\x08duration\x03\0\0\x03\0%wasi:clocks/types@0.3.0-rc-2026-01-06\
\x05\x0e\x02\x03\0\x0b\x08duration\x01B\x0c\x02\x03\x02\x01\x0f\x04\0\x08duratio\
n\x03\0\0\x01w\x04\0\x04mark\x03\0\x02\x01@\0\0\x03\x04\0\x03now\x01\x04\x01@\0\0\
\x01\x04\0\x0eget-resolution\x01\x05\x01C\x01\x04when\x03\x01\0\x04\0\x0await-un\
til\x01\x06\x01C\x01\x08how-long\x01\x01\0\x04\0\x08wait-for\x01\x07\x03\0/wasi:\
clocks/monotonic-clock@0.3.0-rc-2026-01-06\x05\x10\x01B\x08\x02\x03\x02\x01\x0f\x04\
\0\x08duration\x03\0\0\x01r\x02\x07secondsx\x0bnanosecondsy\x04\0\x07instant\x03\
\0\x02\x01@\0\0\x03\x04\0\x03now\x01\x04\x01@\0\0\x01\x04\0\x0eget-resolution\x01\
\x05\x03\0,wasi:clocks/system-clock@0.3.0-rc-2026-01-06\x05\x11\x02\x03\0\x0d\x07\
instant\x01BY\x02\x03\x02\x01\x12\x04\0\x07instant\x03\0\0\x01w\x04\0\x08filesiz\
e\x03\0\x02\x01m\x08\x07unknown\x0cblock-device\x10character-device\x09directory\
\x04fifo\x0dsymbolic-link\x0cregular-file\x06socket\x04\0\x0fdescriptor-type\x03\
\0\x04\x01n\x06\x04read\x05write\x13file-integrity-sync\x13data-integrity-sync\x14\
requested-write-sync\x10mutate-directory\x04\0\x10descriptor-flags\x03\0\x06\x01\
n\x01\x0esymlink-follow\x04\0\x0apath-flags\x03\0\x08\x01n\x04\x06create\x09dire\
ctory\x09exclusive\x08truncate\x04\0\x0aopen-flags\x03\0\x0a\x01w\x04\0\x0alink-\
count\x03\0\x0c\x01k\x01\x01r\x06\x04type\x05\x0alink-count\x0d\x04size\x03\x15d\
ata-access-timestamp\x0e\x1bdata-modification-timestamp\x0e\x17status-change-tim\
estamp\x0e\x04\0\x0fdescriptor-stat\x03\0\x0f\x01q\x03\x09no-change\0\0\x03now\0\
\0\x09timestamp\x01\x01\0\x04\0\x0dnew-timestamp\x03\0\x11\x01r\x02\x04type\x05\x04\
names\x04\0\x0fdirectory-entry\x03\0\x13\x01m$\x06access\x07already\x0ebad-descr\
iptor\x04busy\x08deadlock\x05quota\x05exist\x0efile-too-large\x15illegal-byte-se\
quence\x0bin-progress\x0binterrupted\x07invalid\x02io\x0cis-directory\x04loop\x0e\
too-many-links\x0cmessage-size\x0dname-too-long\x09no-device\x08no-entry\x07no-l\
ock\x13insufficient-memory\x12insufficient-space\x0dnot-directory\x09not-empty\x0f\
not-recoverable\x0bunsupported\x06no-tty\x0eno-such-device\x08overflow\x0dnot-pe\
rmitted\x04pipe\x09read-only\x0cinvalid-seek\x0etext-file-busy\x0ccross-device\x04\
\0\x0aerror-code\x03\0\x15\x01m\x06\x06normal\x0asequential\x06random\x09will-ne\
ed\x09dont-need\x08no-reuse\x04\0\x06advice\x03\0\x17\x01r\x02\x05lowerw\x05uppe\
rw\x04\0\x13metadata-hash-value\x03\0\x19\x04\0\x0adescriptor\x03\x01\x01h\x1b\x01\
f\x01}\x01j\0\x01\x16\x01e\x01\x1e\x01o\x02\x1d\x1f\x01@\x02\x04self\x1c\x06offs\
et\x03\0\x20\x04\0\"[method]descriptor.read-via-stream\x01!\x01C\x03\x04self\x1c\
\x04data\x1d\x06offset\x03\0\x1e\x04\0#[method]descriptor.write-via-stream\x01\"\
\x01C\x02\x04self\x1c\x04data\x1d\0\x1e\x04\0$[method]descriptor.append-via-stre\
am\x01#\x01C\x04\x04self\x1c\x06offset\x03\x06length\x03\x06advice\x18\0\x1e\x04\
\0\x19[method]descriptor.advise\x01$\x01C\x01\x04self\x1c\0\x1e\x04\0\x1c[method\
]descriptor.sync-data\x01%\x01j\x01\x07\x01\x16\x01C\x01\x04self\x1c\0&\x04\0\x1c\
[method]descriptor.get-flags\x01'\x01j\x01\x05\x01\x16\x01C\x01\x04self\x1c\0(\x04\
\0\x1b[method]descriptor.get-type\x01)\x01C\x02\x04self\x1c\x04size\x03\0\x1e\x04\
\0\x1b[method]descriptor.set-size\x01*\x01C\x03\x04self\x1c\x15data-access-times\
tamp\x12\x1bdata-modification-timestamp\x12\0\x1e\x04\0\x1c[method]descriptor.se\
t-times\x01+\x01f\x01\x14\x01o\x02,\x1f\x01C\x01\x04self\x1c\0-\x04\0![method]de\
scriptor.read-directory\x01.\x04\0\x17[method]descriptor.sync\x01%\x01C\x02\x04s\
elf\x1c\x04paths\0\x1e\x04\0&[method]descriptor.create-directory-at\x01/\x01j\x01\
\x10\x01\x16\x01C\x01\x04self\x1c\00\x04\0\x17[method]descriptor.stat\x011\x01C\x03\
\x04self\x1c\x0apath-flags\x09\x04paths\00\x04\0\x1a[method]descriptor.stat-at\x01\
2\x01C\x05\x04self\x1c\x0apath-flags\x09\x04paths\x15data-access-timestamp\x12\x1b\
data-modification-timestamp\x12\0\x1e\x04\0\x1f[method]descriptor.set-times-at\x01\
3\x01C\x05\x04self\x1c\x0eold-path-flags\x09\x08old-paths\x0enew-descriptor\x1c\x08\
new-paths\0\x1e\x04\0\x1a[method]descriptor.link-at\x014\x01i\x1b\x01j\x015\x01\x16\
\x01C\x05\x04self\x1c\x0apath-flags\x09\x04paths\x0aopen-flags\x0b\x05flags\x07\0\
6\x04\0\x1a[method]descriptor.open-at\x017\x01j\x01s\x01\x16\x01C\x02\x04self\x1c\
\x04paths\08\x04\0\x1e[method]descriptor.readlink-at\x019\x04\0&[method]descript\
or.remove-directory-at\x01/\x01C\x04\x04self\x1c\x08old-paths\x0enew-descriptor\x1c\
\x08new-paths\0\x1e\x04\0\x1c[method]descriptor.rename-at\x01:\x01C\x03\x04self\x1c\
\x08old-paths\x08new-paths\0\x1e\x04\0\x1d[method]descriptor.symlink-at\x01;\x04\
\0![method]descriptor.unlink-file-at\x01/\x01C\x02\x04self\x1c\x05other\x1c\0\x7f\
\x04\0![method]descriptor.is-same-object\x01<\x01j\x01\x1a\x01\x16\x01C\x01\x04s\
elf\x1c\0=\x04\0\x20[method]descriptor.metadata-hash\x01>\x01C\x03\x04self\x1c\x0a\
path-flags\x09\x04paths\0=\x04\0#[method]descriptor.metadata-hash-at\x01?\x03\0)\
wasi:filesystem/types@0.3.0-rc-2026-01-06\x05\x13\x02\x03\0\x0e\x0adescriptor\x01\
B\x07\x02\x03\x02\x01\x14\x04\0\x0adescriptor\x03\0\0\x01i\x01\x01o\x02\x02s\x01\
p\x03\x01@\0\0\x04\x04\0\x0fget-directories\x01\x05\x03\0,wasi:filesystem/preope\
ns@0.3.0-rc-2026-01-06\x05\x15\x01Bq\x02\x03\x02\x01\x0f\x04\0\x08duration\x03\0\
\0\x01m\x0e\x07unknown\x0daccess-denied\x0dnot-supported\x10invalid-argument\x0d\
out-of-memory\x07timeout\x0dinvalid-state\x14address-not-bindable\x0eaddress-in-\
use\x12remote-unreachable\x12connection-refused\x10connection-reset\x12connectio\
n-aborted\x12datagram-too-large\x04\0\x0aerror-code\x03\0\x02\x01m\x02\x04ipv4\x04\
ipv6\x04\0\x11ip-address-family\x03\0\x04\x01o\x04}}}}\x04\0\x0cipv4-address\x03\
\0\x06\x01o\x08{{{{{{{{\x04\0\x0cipv6-address\x03\0\x08\x01q\x02\x04ipv4\x01\x07\
\0\x04ipv6\x01\x09\0\x04\0\x0aip-address\x03\0\x0a\x01r\x02\x04port{\x07address\x07\
\x04\0\x13ipv4-socket-address\x03\0\x0c\x01r\x04\x04port{\x09flow-infoy\x07addre\
ss\x09\x08scope-idy\x04\0\x13ipv6-socket-address\x03\0\x0e\x01q\x02\x04ipv4\x01\x0d\
\0\x04ipv6\x01\x0f\0\x04\0\x11ip-socket-address\x03\0\x10\x04\0\x0atcp-socket\x03\
\x01\x04\0\x0audp-socket\x03\x01\x01i\x12\x01j\x01\x14\x01\x03\x01@\x01\x0eaddre\
ss-family\x05\0\x15\x04\0\x19[static]tcp-socket.create\x01\x16\x01h\x12\x01j\0\x01\
\x03\x01@\x02\x04self\x17\x0dlocal-address\x11\0\x18\x04\0\x17[method]tcp-socket\
.bind\x01\x19\x01C\x02\x04self\x17\x0eremote-address\x11\0\x18\x04\0\x1a[method]\
tcp-socket.connect\x01\x1a\x01f\x01\x14\x01j\x01\x1b\x01\x03\x01@\x01\x04self\x17\
\0\x1c\x04\0\x19[method]tcp-socket.listen\x01\x1d\x01f\x01}\x01C\x02\x04self\x17\
\x04data\x1e\0\x18\x04\0\x17[method]tcp-socket.send\x01\x1f\x01e\x01\x18\x01o\x02\
\x1e\x20\x01@\x01\x04self\x17\0!\x04\0\x1a[method]tcp-socket.receive\x01\"\x01j\x01\
\x11\x01\x03\x01@\x01\x04self\x17\0#\x04\0$[method]tcp-socket.get-local-address\x01\
$\x04\0%[method]tcp-socket.get-remote-address\x01$\x01@\x01\x04self\x17\0\x7f\x04\
\0#[method]tcp-socket.get-is-listening\x01%\x01@\x01\x04self\x17\0\x05\x04\0%[me\
thod]tcp-socket.get-address-family\x01&\x01@\x02\x04self\x17\x05valuew\0\x18\x04\
\0*[method]tcp-socket.set-listen-backlog-size\x01'\x01j\x01\x7f\x01\x03\x01@\x01\
\x04self\x17\0(\x04\0)[method]tcp-socket.get-keep-alive-enabled\x01)\x01@\x02\x04\
self\x17\x05value\x7f\0\x18\x04\0)[method]tcp-socket.set-keep-alive-enabled\x01*\
\x01j\x01\x01\x01\x03\x01@\x01\x04self\x17\0+\x04\0+[method]tcp-socket.get-keep-\
alive-idle-time\x01,\x01@\x02\x04self\x17\x05value\x01\0\x18\x04\0+[method]tcp-s\
ocket.set-keep-alive-idle-time\x01-\x04\0*[method]tcp-socket.get-keep-alive-inte\
rval\x01,\x04\0*[method]tcp-socket.set-keep-alive-interval\x01-\x01j\x01y\x01\x03\
\x01@\x01\x04self\x17\0.\x04\0'[method]tcp-socket.get-keep-alive-count\x01/\x01@\
\x02\x04self\x17\x05valuey\0\x18\x04\0'[method]tcp-socket.set-keep-alive-count\x01\
0\x01j\x01}\x01\x03\x01@\x01\x04self\x17\01\x04\0\x20[method]tcp-socket.get-hop-\
limit\x012\x01@\x02\x04self\x17\x05value}\0\x18\x04\0\x20[method]tcp-socket.set-\
hop-limit\x013\x01j\x01w\x01\x03\x01@\x01\x04self\x17\04\x04\0*[method]tcp-socke\
t.get-receive-buffer-size\x015\x04\0*[method]tcp-socket.set-receive-buffer-size\x01\
'\x04\0'[method]tcp-socket.get-send-buffer-size\x015\x04\0'[method]tcp-socket.se\
t-send-buffer-size\x01'\x01i\x13\x01j\x016\x01\x03\x01@\x01\x0eaddress-family\x05\
\07\x04\0\x19[static]udp-socket.create\x018\x01h\x13\x01@\x02\x04self9\x0dlocal-\
address\x11\0\x18\x04\0\x17[method]udp-socket.bind\x01:\x01@\x02\x04self9\x0erem\
ote-address\x11\0\x18\x04\0\x1a[method]udp-socket.connect\x01;\x01@\x01\x04self9\
\0\x18\x04\0\x1d[method]udp-socket.disconnect\x01<\x01p}\x01k\x11\x01C\x03\x04se\
lf9\x04data=\x0eremote-address>\0\x18\x04\0\x17[method]udp-socket.send\x01?\x01o\
\x02=\x11\x01j\x01\xc0\0\x01\x03\x01C\x01\x04self9\0\xc1\0\x04\0\x1a[method]udp-\
socket.receive\x01B\x01@\x01\x04self9\0#\x04\0$[method]udp-socket.get-local-addr\
ess\x01C\x04\0%[method]udp-socket.get-remote-address\x01C\x01@\x01\x04self9\0\x05\
\x04\0%[method]udp-socket.get-address-family\x01D\x01@\x01\x04self9\01\x04\0([me\
thod]udp-socket.get-unicast-hop-limit\x01E\x01@\x02\x04self9\x05value}\0\x18\x04\
\0([method]udp-socket.set-unicast-hop-limit\x01F\x01@\x01\x04self9\04\x04\0*[met\
hod]udp-socket.get-receive-buffer-size\x01G\x01@\x02\x04self9\x05valuew\0\x18\x04\
\0*[method]udp-socket.set-receive-buffer-size\x01H\x04\0'[method]udp-socket.get-\
send-buffer-size\x01G\x04\0'[method]udp-socket.set-send-buffer-size\x01H\x03\0&w\
asi:sockets/types@0.3.0-rc-2026-01-06\x05\x16\x02\x03\0\x10\x0aip-address\x01B\x08\
\x02\x03\x02\x01\x17\x04\0\x0aip-address\x03\0\0\x01m\x06\x07unknown\x0daccess-d\
enied\x10invalid-argument\x11name-unresolvable\x1atemporary-resolver-failure\x1a\
permanent-resolver-failure\x04\0\x0aerror-code\x03\0\x02\x01p\x01\x01j\x01\x04\x01\
\x03\x01C\x01\x04names\0\x05\x04\0\x11resolve-addresses\x01\x06\x03\0/wasi:socke\
ts/ip-name-lookup@0.3.0-rc-2026-01-06\x05\x18\x01B\x05\x01p}\x01@\x01\x03lenw\0\0\
\x04\0\x10get-random-bytes\x01\x01\x01@\0\0w\x04\0\x0eget-random-u64\x01\x02\x03\
\0&wasi:random/random@0.3.0-rc-2026-01-06\x05\x19\x01B\x05\x01p}\x01@\x01\x03len\
w\0\0\x04\0\x19get-insecure-random-bytes\x01\x01\x01@\0\0w\x04\0\x17get-insecure\
-random-u64\x01\x02\x03\0(wasi:random/insecure@0.3.0-rc-2026-01-06\x05\x1a\x01B\x03\
\x01o\x02ww\x01@\0\0\0\x04\0\x11get-insecure-seed\x01\x01\x03\0-wasi:random/inse\
cure-seed@0.3.0-rc-2026-01-06\x05\x1b\x01B\x03\x01j\0\0\x01C\0\0\0\x04\0\x03run\x01\
\x01\x04\0\x20wasi:cli/run@0.3.0-rc-2026-01-06\x05\x1c\x04\0$wasi:cli/command@0.\
3.0-rc-2026-01-06\x04\0\x0b\x0d\x01\0\x07command\x03\0\0\0G\x09producers\x01\x0c\
processed-by\x02\x0dwit-component\x070.244.0\x10wit-bindgen-rust\x060.51.0";
        };
    };
}
#[doc(inline)]
pub use __export_command_impl as _export_command;
#[rustfmt::skip]
#[cfg(target_arch = "wasm32")]
#[unsafe(
    link_section = "component-type:wit-bindgen:0.51.0:wasi:cli@0.3.0-rc-2026-01-06:command-with-all-of-its-exports-removed:encoded worldrust-wasip3-0.4.0+wasi-0.3.0-rc-2026-01-06-from-crates-io-command-world"
)]
#[doc(hidden)]
#[allow(clippy::octal_escapes)]
pub static __WIT_BINDGEN_COMPONENT_TYPE: [u8; 8047] = *b"\
\0asm\x0d\0\x01\0\0\x19\x16wit-component-encoding\x04\0\x07\xd1=\x01A\x02\x01A1\x01\
B\x0a\x01o\x02ss\x01p\0\x01@\0\0\x01\x04\0\x0fget-environment\x01\x02\x01ps\x01@\
\0\0\x03\x04\0\x0dget-arguments\x01\x04\x01ks\x01@\0\0\x05\x04\0\x0fget-initial-\
cwd\x01\x06\x03\0(wasi:cli/environment@0.3.0-rc-2026-01-06\x05\0\x01B\x03\x01j\0\
\0\x01@\x01\x06status\0\x01\0\x04\0\x04exit\x01\x01\x03\0!wasi:cli/exit@0.3.0-rc\
-2026-01-06\x05\x01\x01B\x02\x01m\x03\x02io\x15illegal-byte-sequence\x04pipe\x04\
\0\x0aerror-code\x03\0\0\x03\0\"wasi:cli/types@0.3.0-rc-2026-01-06\x05\x02\x02\x03\
\0\x02\x0aerror-code\x01B\x08\x02\x03\x02\x01\x03\x04\0\x0aerror-code\x03\0\0\x01\
f\x01}\x01j\0\x01\x01\x01e\x01\x03\x01o\x02\x02\x04\x01@\0\0\x05\x04\0\x0fread-v\
ia-stream\x01\x06\x03\0\"wasi:cli/stdin@0.3.0-rc-2026-01-06\x05\x04\x01B\x06\x02\
\x03\x02\x01\x03\x04\0\x0aerror-code\x03\0\0\x01f\x01}\x01j\0\x01\x01\x01C\x01\x04\
data\x02\0\x03\x04\0\x10write-via-stream\x01\x04\x03\0#wasi:cli/stdout@0.3.0-rc-\
2026-01-06\x05\x05\x01B\x06\x02\x03\x02\x01\x03\x04\0\x0aerror-code\x03\0\0\x01f\
\x01}\x01j\0\x01\x01\x01C\x01\x04data\x02\0\x03\x04\0\x10write-via-stream\x01\x04\
\x03\0#wasi:cli/stderr@0.3.0-rc-2026-01-06\x05\x06\x01B\x01\x04\0\x0eterminal-in\
put\x03\x01\x03\0+wasi:cli/terminal-input@0.3.0-rc-2026-01-06\x05\x07\x01B\x01\x04\
\0\x0fterminal-output\x03\x01\x03\0,wasi:cli/terminal-output@0.3.0-rc-2026-01-06\
\x05\x08\x02\x03\0\x06\x0eterminal-input\x01B\x06\x02\x03\x02\x01\x09\x04\0\x0et\
erminal-input\x03\0\0\x01i\x01\x01k\x02\x01@\0\0\x03\x04\0\x12get-terminal-stdin\
\x01\x04\x03\0+wasi:cli/terminal-stdin@0.3.0-rc-2026-01-06\x05\x0a\x02\x03\0\x07\
\x0fterminal-output\x01B\x06\x02\x03\x02\x01\x0b\x04\0\x0fterminal-output\x03\0\0\
\x01i\x01\x01k\x02\x01@\0\0\x03\x04\0\x13get-terminal-stdout\x01\x04\x03\0,wasi:\
cli/terminal-stdout@0.3.0-rc-2026-01-06\x05\x0c\x01B\x06\x02\x03\x02\x01\x0b\x04\
\0\x0fterminal-output\x03\0\0\x01i\x01\x01k\x02\x01@\0\0\x03\x04\0\x13get-termin\
al-stderr\x01\x04\x03\0,wasi:cli/terminal-stderr@0.3.0-rc-2026-01-06\x05\x0d\x01\
B\x02\x01w\x04\0\x08duration\x03\0\0\x03\0%wasi:clocks/types@0.3.0-rc-2026-01-06\
\x05\x0e\x02\x03\0\x0b\x08duration\x01B\x0c\x02\x03\x02\x01\x0f\x04\0\x08duratio\
n\x03\0\0\x01w\x04\0\x04mark\x03\0\x02\x01@\0\0\x03\x04\0\x03now\x01\x04\x01@\0\0\
\x01\x04\0\x0eget-resolution\x01\x05\x01C\x01\x04when\x03\x01\0\x04\0\x0await-un\
til\x01\x06\x01C\x01\x08how-long\x01\x01\0\x04\0\x08wait-for\x01\x07\x03\0/wasi:\
clocks/monotonic-clock@0.3.0-rc-2026-01-06\x05\x10\x01B\x08\x02\x03\x02\x01\x0f\x04\
\0\x08duration\x03\0\0\x01r\x02\x07secondsx\x0bnanosecondsy\x04\0\x07instant\x03\
\0\x02\x01@\0\0\x03\x04\0\x03now\x01\x04\x01@\0\0\x01\x04\0\x0eget-resolution\x01\
\x05\x03\0,wasi:clocks/system-clock@0.3.0-rc-2026-01-06\x05\x11\x02\x03\0\x0d\x07\
instant\x01BY\x02\x03\x02\x01\x12\x04\0\x07instant\x03\0\0\x01w\x04\0\x08filesiz\
e\x03\0\x02\x01m\x08\x07unknown\x0cblock-device\x10character-device\x09directory\
\x04fifo\x0dsymbolic-link\x0cregular-file\x06socket\x04\0\x0fdescriptor-type\x03\
\0\x04\x01n\x06\x04read\x05write\x13file-integrity-sync\x13data-integrity-sync\x14\
requested-write-sync\x10mutate-directory\x04\0\x10descriptor-flags\x03\0\x06\x01\
n\x01\x0esymlink-follow\x04\0\x0apath-flags\x03\0\x08\x01n\x04\x06create\x09dire\
ctory\x09exclusive\x08truncate\x04\0\x0aopen-flags\x03\0\x0a\x01w\x04\0\x0alink-\
count\x03\0\x0c\x01k\x01\x01r\x06\x04type\x05\x0alink-count\x0d\x04size\x03\x15d\
ata-access-timestamp\x0e\x1bdata-modification-timestamp\x0e\x17status-change-tim\
estamp\x0e\x04\0\x0fdescriptor-stat\x03\0\x0f\x01q\x03\x09no-change\0\0\x03now\0\
\0\x09timestamp\x01\x01\0\x04\0\x0dnew-timestamp\x03\0\x11\x01r\x02\x04type\x05\x04\
names\x04\0\x0fdirectory-entry\x03\0\x13\x01m$\x06access\x07already\x0ebad-descr\
iptor\x04busy\x08deadlock\x05quota\x05exist\x0efile-too-large\x15illegal-byte-se\
quence\x0bin-progress\x0binterrupted\x07invalid\x02io\x0cis-directory\x04loop\x0e\
too-many-links\x0cmessage-size\x0dname-too-long\x09no-device\x08no-entry\x07no-l\
ock\x13insufficient-memory\x12insufficient-space\x0dnot-directory\x09not-empty\x0f\
not-recoverable\x0bunsupported\x06no-tty\x0eno-such-device\x08overflow\x0dnot-pe\
rmitted\x04pipe\x09read-only\x0cinvalid-seek\x0etext-file-busy\x0ccross-device\x04\
\0\x0aerror-code\x03\0\x15\x01m\x06\x06normal\x0asequential\x06random\x09will-ne\
ed\x09dont-need\x08no-reuse\x04\0\x06advice\x03\0\x17\x01r\x02\x05lowerw\x05uppe\
rw\x04\0\x13metadata-hash-value\x03\0\x19\x04\0\x0adescriptor\x03\x01\x01h\x1b\x01\
f\x01}\x01j\0\x01\x16\x01e\x01\x1e\x01o\x02\x1d\x1f\x01@\x02\x04self\x1c\x06offs\
et\x03\0\x20\x04\0\"[method]descriptor.read-via-stream\x01!\x01C\x03\x04self\x1c\
\x04data\x1d\x06offset\x03\0\x1e\x04\0#[method]descriptor.write-via-stream\x01\"\
\x01C\x02\x04self\x1c\x04data\x1d\0\x1e\x04\0$[method]descriptor.append-via-stre\
am\x01#\x01C\x04\x04self\x1c\x06offset\x03\x06length\x03\x06advice\x18\0\x1e\x04\
\0\x19[method]descriptor.advise\x01$\x01C\x01\x04self\x1c\0\x1e\x04\0\x1c[method\
]descriptor.sync-data\x01%\x01j\x01\x07\x01\x16\x01C\x01\x04self\x1c\0&\x04\0\x1c\
[method]descriptor.get-flags\x01'\x01j\x01\x05\x01\x16\x01C\x01\x04self\x1c\0(\x04\
\0\x1b[method]descriptor.get-type\x01)\x01C\x02\x04self\x1c\x04size\x03\0\x1e\x04\
\0\x1b[method]descriptor.set-size\x01*\x01C\x03\x04self\x1c\x15data-access-times\
tamp\x12\x1bdata-modification-timestamp\x12\0\x1e\x04\0\x1c[method]descriptor.se\
t-times\x01+\x01f\x01\x14\x01o\x02,\x1f\x01C\x01\x04self\x1c\0-\x04\0![method]de\
scriptor.read-directory\x01.\x04\0\x17[method]descriptor.sync\x01%\x01C\x02\x04s\
elf\x1c\x04paths\0\x1e\x04\0&[method]descriptor.create-directory-at\x01/\x01j\x01\
\x10\x01\x16\x01C\x01\x04self\x1c\00\x04\0\x17[method]descriptor.stat\x011\x01C\x03\
\x04self\x1c\x0apath-flags\x09\x04paths\00\x04\0\x1a[method]descriptor.stat-at\x01\
2\x01C\x05\x04self\x1c\x0apath-flags\x09\x04paths\x15data-access-timestamp\x12\x1b\
data-modification-timestamp\x12\0\x1e\x04\0\x1f[method]descriptor.set-times-at\x01\
3\x01C\x05\x04self\x1c\x0eold-path-flags\x09\x08old-paths\x0enew-descriptor\x1c\x08\
new-paths\0\x1e\x04\0\x1a[method]descriptor.link-at\x014\x01i\x1b\x01j\x015\x01\x16\
\x01C\x05\x04self\x1c\x0apath-flags\x09\x04paths\x0aopen-flags\x0b\x05flags\x07\0\
6\x04\0\x1a[method]descriptor.open-at\x017\x01j\x01s\x01\x16\x01C\x02\x04self\x1c\
\x04paths\08\x04\0\x1e[method]descriptor.readlink-at\x019\x04\0&[method]descript\
or.remove-directory-at\x01/\x01C\x04\x04self\x1c\x08old-paths\x0enew-descriptor\x1c\
\x08new-paths\0\x1e\x04\0\x1c[method]descriptor.rename-at\x01:\x01C\x03\x04self\x1c\
\x08old-paths\x08new-paths\0\x1e\x04\0\x1d[method]descriptor.symlink-at\x01;\x04\
\0![method]descriptor.unlink-file-at\x01/\x01C\x02\x04self\x1c\x05other\x1c\0\x7f\
\x04\0![method]descriptor.is-same-object\x01<\x01j\x01\x1a\x01\x16\x01C\x01\x04s\
elf\x1c\0=\x04\0\x20[method]descriptor.metadata-hash\x01>\x01C\x03\x04self\x1c\x0a\
path-flags\x09\x04paths\0=\x04\0#[method]descriptor.metadata-hash-at\x01?\x03\0)\
wasi:filesystem/types@0.3.0-rc-2026-01-06\x05\x13\x02\x03\0\x0e\x0adescriptor\x01\
B\x07\x02\x03\x02\x01\x14\x04\0\x0adescriptor\x03\0\0\x01i\x01\x01o\x02\x02s\x01\
p\x03\x01@\0\0\x04\x04\0\x0fget-directories\x01\x05\x03\0,wasi:filesystem/preope\
ns@0.3.0-rc-2026-01-06\x05\x15\x01Bq\x02\x03\x02\x01\x0f\x04\0\x08duration\x03\0\
\0\x01m\x0e\x07unknown\x0daccess-denied\x0dnot-supported\x10invalid-argument\x0d\
out-of-memory\x07timeout\x0dinvalid-state\x14address-not-bindable\x0eaddress-in-\
use\x12remote-unreachable\x12connection-refused\x10connection-reset\x12connectio\
n-aborted\x12datagram-too-large\x04\0\x0aerror-code\x03\0\x02\x01m\x02\x04ipv4\x04\
ipv6\x04\0\x11ip-address-family\x03\0\x04\x01o\x04}}}}\x04\0\x0cipv4-address\x03\
\0\x06\x01o\x08{{{{{{{{\x04\0\x0cipv6-address\x03\0\x08\x01q\x02\x04ipv4\x01\x07\
\0\x04ipv6\x01\x09\0\x04\0\x0aip-address\x03\0\x0a\x01r\x02\x04port{\x07address\x07\
\x04\0\x13ipv4-socket-address\x03\0\x0c\x01r\x04\x04port{\x09flow-infoy\x07addre\
ss\x09\x08scope-idy\x04\0\x13ipv6-socket-address\x03\0\x0e\x01q\x02\x04ipv4\x01\x0d\
\0\x04ipv6\x01\x0f\0\x04\0\x11ip-socket-address\x03\0\x10\x04\0\x0atcp-socket\x03\
\x01\x04\0\x0audp-socket\x03\x01\x01i\x12\x01j\x01\x14\x01\x03\x01@\x01\x0eaddre\
ss-family\x05\0\x15\x04\0\x19[static]tcp-socket.create\x01\x16\x01h\x12\x01j\0\x01\
\x03\x01@\x02\x04self\x17\x0dlocal-address\x11\0\x18\x04\0\x17[method]tcp-socket\
.bind\x01\x19\x01C\x02\x04self\x17\x0eremote-address\x11\0\x18\x04\0\x1a[method]\
tcp-socket.connect\x01\x1a\x01f\x01\x14\x01j\x01\x1b\x01\x03\x01@\x01\x04self\x17\
\0\x1c\x04\0\x19[method]tcp-socket.listen\x01\x1d\x01f\x01}\x01C\x02\x04self\x17\
\x04data\x1e\0\x18\x04\0\x17[method]tcp-socket.send\x01\x1f\x01e\x01\x18\x01o\x02\
\x1e\x20\x01@\x01\x04self\x17\0!\x04\0\x1a[method]tcp-socket.receive\x01\"\x01j\x01\
\x11\x01\x03\x01@\x01\x04self\x17\0#\x04\0$[method]tcp-socket.get-local-address\x01\
$\x04\0%[method]tcp-socket.get-remote-address\x01$\x01@\x01\x04self\x17\0\x7f\x04\
\0#[method]tcp-socket.get-is-listening\x01%\x01@\x01\x04self\x17\0\x05\x04\0%[me\
thod]tcp-socket.get-address-family\x01&\x01@\x02\x04self\x17\x05valuew\0\x18\x04\
\0*[method]tcp-socket.set-listen-backlog-size\x01'\x01j\x01\x7f\x01\x03\x01@\x01\
\x04self\x17\0(\x04\0)[method]tcp-socket.get-keep-alive-enabled\x01)\x01@\x02\x04\
self\x17\x05value\x7f\0\x18\x04\0)[method]tcp-socket.set-keep-alive-enabled\x01*\
\x01j\x01\x01\x01\x03\x01@\x01\x04self\x17\0+\x04\0+[method]tcp-socket.get-keep-\
alive-idle-time\x01,\x01@\x02\x04self\x17\x05value\x01\0\x18\x04\0+[method]tcp-s\
ocket.set-keep-alive-idle-time\x01-\x04\0*[method]tcp-socket.get-keep-alive-inte\
rval\x01,\x04\0*[method]tcp-socket.set-keep-alive-interval\x01-\x01j\x01y\x01\x03\
\x01@\x01\x04self\x17\0.\x04\0'[method]tcp-socket.get-keep-alive-count\x01/\x01@\
\x02\x04self\x17\x05valuey\0\x18\x04\0'[method]tcp-socket.set-keep-alive-count\x01\
0\x01j\x01}\x01\x03\x01@\x01\x04self\x17\01\x04\0\x20[method]tcp-socket.get-hop-\
limit\x012\x01@\x02\x04self\x17\x05value}\0\x18\x04\0\x20[method]tcp-socket.set-\
hop-limit\x013\x01j\x01w\x01\x03\x01@\x01\x04self\x17\04\x04\0*[method]tcp-socke\
t.get-receive-buffer-size\x015\x04\0*[method]tcp-socket.set-receive-buffer-size\x01\
'\x04\0'[method]tcp-socket.get-send-buffer-size\x015\x04\0'[method]tcp-socket.se\
t-send-buffer-size\x01'\x01i\x13\x01j\x016\x01\x03\x01@\x01\x0eaddress-family\x05\
\07\x04\0\x19[static]udp-socket.create\x018\x01h\x13\x01@\x02\x04self9\x0dlocal-\
address\x11\0\x18\x04\0\x17[method]udp-socket.bind\x01:\x01@\x02\x04self9\x0erem\
ote-address\x11\0\x18\x04\0\x1a[method]udp-socket.connect\x01;\x01@\x01\x04self9\
\0\x18\x04\0\x1d[method]udp-socket.disconnect\x01<\x01p}\x01k\x11\x01C\x03\x04se\
lf9\x04data=\x0eremote-address>\0\x18\x04\0\x17[method]udp-socket.send\x01?\x01o\
\x02=\x11\x01j\x01\xc0\0\x01\x03\x01C\x01\x04self9\0\xc1\0\x04\0\x1a[method]udp-\
socket.receive\x01B\x01@\x01\x04self9\0#\x04\0$[method]udp-socket.get-local-addr\
ess\x01C\x04\0%[method]udp-socket.get-remote-address\x01C\x01@\x01\x04self9\0\x05\
\x04\0%[method]udp-socket.get-address-family\x01D\x01@\x01\x04self9\01\x04\0([me\
thod]udp-socket.get-unicast-hop-limit\x01E\x01@\x02\x04self9\x05value}\0\x18\x04\
\0([method]udp-socket.set-unicast-hop-limit\x01F\x01@\x01\x04self9\04\x04\0*[met\
hod]udp-socket.get-receive-buffer-size\x01G\x01@\x02\x04self9\x05valuew\0\x18\x04\
\0*[method]udp-socket.set-receive-buffer-size\x01H\x04\0'[method]udp-socket.get-\
send-buffer-size\x01G\x04\0'[method]udp-socket.set-send-buffer-size\x01H\x03\0&w\
asi:sockets/types@0.3.0-rc-2026-01-06\x05\x16\x02\x03\0\x10\x0aip-address\x01B\x08\
\x02\x03\x02\x01\x17\x04\0\x0aip-address\x03\0\0\x01m\x06\x07unknown\x0daccess-d\
enied\x10invalid-argument\x11name-unresolvable\x1atemporary-resolver-failure\x1a\
permanent-resolver-failure\x04\0\x0aerror-code\x03\0\x02\x01p\x01\x01j\x01\x04\x01\
\x03\x01C\x01\x04names\0\x05\x04\0\x11resolve-addresses\x01\x06\x03\0/wasi:socke\
ts/ip-name-lookup@0.3.0-rc-2026-01-06\x05\x18\x01B\x05\x01p}\x01@\x01\x03lenw\0\0\
\x04\0\x10get-random-bytes\x01\x01\x01@\0\0w\x04\0\x0eget-random-u64\x01\x02\x03\
\0&wasi:random/random@0.3.0-rc-2026-01-06\x05\x19\x01B\x05\x01p}\x01@\x01\x03len\
w\0\0\x04\0\x19get-insecure-random-bytes\x01\x01\x01@\0\0w\x04\0\x17get-insecure\
-random-u64\x01\x02\x03\0(wasi:random/insecure@0.3.0-rc-2026-01-06\x05\x1a\x01B\x03\
\x01o\x02ww\x01@\0\0\0\x04\0\x11get-insecure-seed\x01\x01\x03\0-wasi:random/inse\
cure-seed@0.3.0-rc-2026-01-06\x05\x1b\x04\0Dwasi:cli/command-with-all-of-its-exp\
orts-removed@0.3.0-rc-2026-01-06\x04\0\x0b-\x01\0'command-with-all-of-its-export\
s-removed\x03\0\0\0G\x09producers\x01\x0cprocessed-by\x02\x0dwit-component\x070.\
244.0\x10wit-bindgen-rust\x060.51.0";
#[inline(never)]
#[doc(hidden)]
pub fn __link_custom_section_describing_imports() {
    wit_bindgen::rt::maybe_link_cabi_realloc();
}
