// Copyright (c) 2024, Google Inc.
// SPDX-License-Identifier: ISC

#ifndef OPENSSL_HEADER_TIME_H
#define OPENSSL_HEADER_TIME_H

// Compatibility header, to be deprecated. use <openssl/posix_time.h> instead.

#include <openssl/posix_time.h>

#endif  // OPENSSL_HEADER_TIME_H
